@@ -31,12 +31,13 @@
 
 /* clutter-actor.h */
 
-#include <glib-object.h>
 #include <pango/pango.h>
 #include <atk/atk.h>
 
-#include <clutter/clutter-color.h>
 #include <clutter/clutter-types.h>
+
+#include <clutter/clutter-color.h>
+#include <clutter/clutter-effect.h>
 #include <clutter/clutter-event.h>
 #include <clutter/clutter-shader.h>
 
@@ -228,8 +229,8 @@ struct _ClutterActor
  * @enter_event: signal class closure for #ClutterActor::enter-event
  * @leave_event: signal class closure for #ClutterActor::leave-event
  * @captured_event: signal class closure for #ClutterActor::captured-event
- * @key_focus_in: signal class closure for #ClutterActor::focus-in
- * @key_focus_out: signal class closure for #ClutterActor::focus-out
+ * @key_focus_in: signal class closure for #ClutterActor::key-focus-in
+ * @key_focus_out: signal class closure for #ClutterActor::key-focus-out
  * @queue_relayout: class handler for #ClutterActor::queue-relayout
  *
  * Base class for actors.
@@ -276,9 +277,6 @@ struct _ClutterActorClass
   void (* apply_transform)      (ClutterActor           *actor,
                                  CoglMatrix             *matrix);
 
-  /* accessibility support */
-  AtkObject* (*get_accessible)  (ClutterActor *actor);
-
   /* event signals */
   gboolean (* event)                (ClutterActor         *actor,
                                      ClutterEvent         *event);
@@ -305,6 +303,9 @@ struct _ClutterActorClass
 
   void     (* queue_relayout)       (ClutterActor         *actor);
 
+  /* accessibility support */
+  AtkObject * (* get_accessible)    (ClutterActor         *actor);
+
   /*< private >*/
   /* padding for future expansion */
   gpointer _padding_dummy[30];
@@ -360,6 +361,13 @@ void                  clutter_actor_allocate_available_size   (ClutterActor
                                                                gfloat                 available_width,
                                                                gfloat                 available_height,
                                                                ClutterAllocationFlags flags);
+void                  clutter_actor_allocate_align_fill       (ClutterActor          *self,
+                                                               const ClutterActorBox *box,
+                                                               gdouble                x_align,
+                                                               gdouble                y_align,
+                                                               gboolean               x_fill,
+                                                               gboolean               y_fill,
+                                                               ClutterAllocationFlags flags);
 void                  clutter_actor_get_allocation_box        (ClutterActor          *self,
                                                                ClutterActorBox       *box);
 void                  clutter_actor_get_allocation_geometry   (ClutterActor          *self,
@@ -447,6 +455,9 @@ void                  clutter_actor_get_clip                  (ClutterActor
                                                                gfloat                *yoff,
                                                                gfloat                *width,
                                                                gfloat                *height);
+void                  clutter_actor_set_clip_to_allocation    (ClutterActor          *self,
+                                                               gboolean               clip_set);
+gboolean              clutter_actor_get_clip_to_allocation    (ClutterActor          *self);
 
 void                  clutter_actor_set_parent                (ClutterActor          *self,
                                                                ClutterActor          *parent);
@@ -454,6 +465,8 @@ ClutterActor *        clutter_actor_get_parent                (ClutterActor
 void                  clutter_actor_reparent                  (ClutterActor          *self,
                                                                ClutterActor          *new_parent);
 void                  clutter_actor_unparent                  (ClutterActor          *self);
+gboolean              clutter_actor_contains                  (ClutterActor          *self,
+							       ClutterActor          *descendant);
 ClutterActor*         clutter_actor_get_stage                 (ClutterActor          *actor);
 
 void                  clutter_actor_raise                     (ClutterActor          *self,
@@ -570,7 +583,9 @@ void                 clutter_actor_pop_internal       (ClutterActor         *sel
 
 gboolean             clutter_actor_has_allocation     (ClutterActor         *self);
 
-AtkObject*           clutter_actor_get_accessible     (ClutterActor *actor);
+AtkObject *          clutter_actor_get_accessible     (ClutterActor         *self);
+
+gboolean             clutter_actor_has_key_focus      (ClutterActor         *self);
 
 G_END_DECLS
 
