@@ -85,30 +85,41 @@ struct _ClutterBackendClass
                                                 ClutterStage    *stage);
   ClutterDeviceManager *(* get_device_manager) (ClutterBackend  *backend);
 
+  void                  (* copy_event_data)    (ClutterBackend     *backend,
+                                                const ClutterEvent *src,
+                                                ClutterEvent       *dest);
+  void                  (* free_event_data)    (ClutterBackend     *backend,
+                                                ClutterEvent       *event);
+
   /* signals */
   void (* resolution_changed) (ClutterBackend *backend);
   void (* font_changed)       (ClutterBackend *backend);
+  void (* settings_changed)   (ClutterBackend *backend);
 };
 
 GType clutter_backend_get_type    (void) G_GNUC_CONST;
 
 ClutterBackend *clutter_get_default_backend (void);
 
+#ifndef CLUTTER_DISABLE_DEPRECATED
 void                        clutter_backend_set_resolution            (ClutterBackend             *backend,
                                                                        gdouble                     dpi);
-gdouble                     clutter_backend_get_resolution            (ClutterBackend             *backend);
 void                        clutter_backend_set_double_click_time     (ClutterBackend             *backend,
                                                                        guint                       msec);
 guint                       clutter_backend_get_double_click_time     (ClutterBackend             *backend);
 void                        clutter_backend_set_double_click_distance (ClutterBackend             *backend,
                                                                        guint                       distance);
 guint                       clutter_backend_get_double_click_distance (ClutterBackend             *backend);
-void                        clutter_backend_set_font_options          (ClutterBackend             *backend,
-                                                                       const cairo_font_options_t *options);
-const cairo_font_options_t *clutter_backend_get_font_options          (ClutterBackend             *backend);
 void                        clutter_backend_set_font_name             (ClutterBackend             *backend,
                                                                        const gchar                *font_name);
 G_CONST_RETURN gchar *      clutter_backend_get_font_name             (ClutterBackend             *backend);
+#endif /* CLUTTER_DISABLE_DEPRECATED */
+
+gdouble                     clutter_backend_get_resolution            (ClutterBackend             *backend);
+
+void                        clutter_backend_set_font_options          (ClutterBackend             *backend,
+                                                                       const cairo_font_options_t *options);
+const cairo_font_options_t *clutter_backend_get_font_options          (ClutterBackend             *backend);
 
 G_END_DECLS
 
