@@ -43,6 +43,52 @@
  * be synthesized by Clutter itself or by the application code.
  */
 
+typedef struct _ClutterEventPrivate {
+  ClutterEvent base;
+
+  gpointer platform_data;
+} ClutterEventPrivate;
+
+static GHashTable *all_events = NULL;
+
+static gboolean
+is_event_allocated (const ClutterEvent *event)
+{
+  if (all_events == NULL)
+    return FALSE;
+
+  return g_hash_table_lookup (all_events, event) != NULL;
+}
+
+/*
+ * _clutter_event_get_platform_data:
+ * @event: a #ClutterEvent
+ *
+ * Retrieves the pointer to platform-specific data inside an event
+ *
+ * Return value: a pointer to platform-specific data
+ *
+ * Since: 1.4
+ */
+gpointer
+_clutter_event_get_platform_data (const ClutterEvent *event)
+{
+  if (!is_event_allocated (event))
+    return NULL;
+
+  return ((ClutterEventPrivate *) event)->platform_data;
+}
+
+void
+_clutter_event_set_platform_data (ClutterEvent *event,
+                                  gpointer      data)
+{
+  if (!is_event_allocated (event))
+    return;
+
+  ((ClutterEventPrivate *) event)->platform_data = data;
+}
+
 /**
  * clutter_event_type:
  * @event: a #ClutterEvent
@@ -52,7 +98,7 @@
  * Return value: a #ClutterEventType
  */
 ClutterEventType
-clutter_event_type (ClutterEvent *event)
+clutter_event_type (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, CLUTTER_NOTHING);
 
@@ -70,7 +116,7 @@ clutter_event_type (ClutterEvent *event)
  * Since: 0.4
  */
 guint32
-clutter_event_get_time (ClutterEvent *event)
+clutter_event_get_time (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, CLUTTER_CURRENT_TIME);
 
@@ -88,7 +134,7 @@ clutter_event_get_time (ClutterEvent *event)
  * Since: 0.4
  */
 ClutterModifierType
-clutter_event_get_state (ClutterEvent *event)
+clutter_event_get_state (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, 0);
 
@@ -118,17 +164,17 @@ clutter_event_get_state (ClutterEvent *event)
 /**
  * clutter_event_get_coords:
  * @event: a #ClutterEvent
- * @x: return location for the X coordinate
- * @y: return location for the Y coordinate
+ * @x: (out): return location for the X coordinate, or %NULL
+ * @y: (out): return location for the Y coordinate, or %NULL
  *
  * Retrieves the coordinates of @event and puts them into @x and @y.
  *
  * Since: 0.4
  */
 void
-clutter_event_get_coords (ClutterEvent *event,
-                          gfloat       *x,
-                          gfloat       *y)
+clutter_event_get_coords (const ClutterEvent *event,
+                          gfloat             *x,
+                          gfloat             *y)
 {
   gfloat event_x, event_y;
 
@@ -189,7 +235,7 @@ clutter_event_get_coords (ClutterEvent *event,
  * Since: 0.6
  */
 ClutterActor *
-clutter_event_get_source (ClutterEvent *event)
+clutter_event_get_source (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, NULL);
 
@@ -208,7 +254,7 @@ clutter_event_get_source (ClutterEvent *event)
  * Since: 0.8
  */
 ClutterStage *
-clutter_event_get_stage (ClutterEvent *event)
+clutter_event_get_stage (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, NULL);
 
@@ -226,7 +272,7 @@ clutter_event_get_stage (ClutterEvent *event)
  * Since: 1.0
  */
 ClutterEventFlags
-clutter_event_get_flags (ClutterEvent *event)
+clutter_event_get_flags (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, CLUTTER_EVENT_NONE);
 
@@ -245,7 +291,7 @@ clutter_event_get_flags (ClutterEvent *event)
  * Since: 1.0
  */
 ClutterActor *
-clutter_event_get_related (ClutterEvent *event)
+clutter_event_get_related (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, NULL);
   g_return_val_if_fail (event->type == CLUTTER_ENTER ||
@@ -265,7 +311,7 @@ clutter_event_get_related (ClutterEvent *event)
  * Since: 1.0
  */
 ClutterScrollDirection
-clutter_event_get_scroll_direction (ClutterEvent *event)
+clutter_event_get_scroll_direction (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, CLUTTER_SCROLL_UP);
   g_return_val_if_fail (event->type == CLUTTER_SCROLL, CLUTTER_SCROLL_UP);
@@ -285,7 +331,7 @@ clutter_event_get_scroll_direction (ClutterEvent *event)
  * Since: 1.0
  */
 guint32
-clutter_event_get_button (ClutterEvent *event)
+clutter_event_get_button (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, 0);
   g_return_val_if_fail (event->type == CLUTTER_BUTTON_PRESS ||
@@ -306,7 +352,7 @@ clutter_event_get_button (ClutterEvent *event)
  * Since: 1.0
  */
 guint32
-clutter_event_get_click_count (ClutterEvent *event)
+clutter_event_get_click_count (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, 0);
   g_return_val_if_fail (event->type == CLUTTER_BUTTON_PRESS ||
@@ -329,7 +375,7 @@ clutter_event_get_click_count (ClutterEvent *event)
  * Since: 1.0
  */
 guint
-clutter_event_get_key_symbol (ClutterEvent *event)
+clutter_event_get_key_symbol (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, 0);
   g_return_val_if_fail (event->type == CLUTTER_KEY_PRESS ||
@@ -350,7 +396,7 @@ clutter_event_get_key_symbol (ClutterEvent *event)
  * Since: 1.0
  */
 guint16
-clutter_event_get_key_code (ClutterEvent *event)
+clutter_event_get_key_code (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, 0);
   g_return_val_if_fail (event->type == CLUTTER_KEY_PRESS ||
@@ -368,7 +414,7 @@ clutter_event_get_key_code (ClutterEvent *event)
  * Return value: The unicode value representing the key
  */
 guint32
-clutter_event_get_key_unicode (ClutterEvent *event)
+clutter_event_get_key_unicode (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, 0);
   g_return_val_if_fail (event->type == CLUTTER_KEY_PRESS ||
@@ -437,7 +483,7 @@ clutter_keysym_to_unicode (guint keyval)
  *   no specific device set.
  */
 gint
-clutter_event_get_device_id (ClutterEvent *event)
+clutter_event_get_device_id (const ClutterEvent *event)
 {
   ClutterInputDevice *device = NULL;
 
@@ -487,7 +533,7 @@ clutter_event_get_device_id (ClutterEvent *event)
  * Since: 1.0
  */
 ClutterInputDeviceType
-clutter_event_get_device_type (ClutterEvent *event)
+clutter_event_get_device_type (const ClutterEvent *event)
 {
   ClutterInputDevice *device = NULL;
 
@@ -538,12 +584,14 @@ clutter_event_get_device_type (ClutterEvent *event)
  * The #ClutterInputDevice structure is completely opaque and should
  * be cast to the platform-specific implementation.
  *
- * Return value: the #ClutterInputDevice or %NULL
+ * Return value: (transfer none): the #ClutterInputDevice or %NULL. The
+ *   returned device is owned by the #ClutterEvent and it should not
+ *   be unreferenced
  *
  * Since: 1.0
  */
 ClutterInputDevice *
-clutter_event_get_device (ClutterEvent *event)
+clutter_event_get_device (const ClutterEvent *event)
 {
   ClutterInputDevice *device = NULL;
 
@@ -606,10 +654,18 @@ ClutterEvent *
 clutter_event_new (ClutterEventType type)
 {
   ClutterEvent *new_event;
+  ClutterEventPrivate *priv;
 
-  new_event = g_slice_new0 (ClutterEvent);
+  priv = g_slice_new0 (ClutterEventPrivate);
+
+  new_event = (ClutterEvent *) priv;
   new_event->type = new_event->any.type = type;
 
+  if (G_UNLIKELY (all_events == NULL))
+    all_events = g_hash_table_new (NULL, NULL);
+
+  g_hash_table_replace (all_events, priv, GUINT_TO_POINTER (1));
+
   return new_event;
 }
 
@@ -622,7 +678,7 @@ clutter_event_new (ClutterEventType type)
  * Return value: A newly allocated #ClutterEvent
  */
 ClutterEvent *
-clutter_event_copy (ClutterEvent *event)
+clutter_event_copy (const ClutterEvent *event)
 {
   ClutterEvent *new_event;
 
@@ -631,6 +687,11 @@ clutter_event_copy (ClutterEvent *event)
   new_event = clutter_event_new (CLUTTER_NOTHING);
   *new_event = *event;
 
+  if (is_event_allocated (event))
+    _clutter_backend_copy_event_data (clutter_get_default_backend (),
+                                      event,
+                                      new_event);
+
   return new_event;
 }
 
@@ -644,7 +705,12 @@ void
 clutter_event_free (ClutterEvent *event)
 {
   if (G_LIKELY (event != NULL))
-    g_slice_free (ClutterEvent, event);
+    {
+      _clutter_backend_free_event_data (clutter_get_default_backend (), event);
+
+      g_hash_table_remove (all_events, event);
+      g_slice_free (ClutterEventPrivate, (ClutterEventPrivate *) event);
+    }
 }
 
 /**
@@ -710,7 +776,7 @@ clutter_event_peek (void)
  * Since: 0.6
  */
 void
-clutter_event_put (ClutterEvent *event)
+clutter_event_put (const ClutterEvent *event)
 {
   ClutterMainContext *context = _clutter_context_get_default ();
   ClutterEvent       *event_copy;
