@@ -217,9 +217,13 @@ enum
   PROP_0,
 
   PROP_FILENAME_SET,
-  PROP_FILENAME
+  PROP_FILENAME,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 #define CLUTTER_SCRIPT_GET_PRIVATE(obj) \
         (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CLUTTER_TYPE_SCRIPT, ClutterScriptPrivate))
 
@@ -397,10 +401,11 @@ clutter_script_class_init (ClutterScriptClass *klass)
    * Since: 0.6
    */
   pspec = g_param_spec_boolean ("filename-set",
-                                "Filename Set",
-                                "Whether the :filename property is set",
+                                P_("Filename Set"),
+                                P_("Whether the :filename property is set"),
                                 FALSE,
                                 CLUTTER_PARAM_READABLE);
+  obj_props[PROP_FILENAME_SET] = pspec;
   g_object_class_install_property (gobject_class, PROP_FILENAME_SET, pspec);
 
   /**
@@ -412,10 +417,11 @@ clutter_script_class_init (ClutterScriptClass *klass)
    * Since: 0.6
    */
   pspec = g_param_spec_string ("filename",
-                               "Filename",
-                               "The path of the currently parsed file",
+                               P_("Filename"),
+                               P_("The path of the currently parsed file"),
                                NULL,
                                CLUTTER_PARAM_READABLE);
+  obj_props[PROP_FILENAME] = pspec;
   g_object_class_install_property (gobject_class, PROP_FILENAME, pspec);
 }
 
@@ -468,7 +474,7 @@ clutter_script_new (void)
  *
  * Return value: on error, zero is returned and @error is set
  *   accordingly. On success, the merge id for the UI definitions is
- *   returned. You can use the merge id with clutter_script_unmerge().
+ *   returned. You can use the merge id with clutter_script_unmerge_objects().
  *
  * Since: 0.6
  */
@@ -978,7 +984,8 @@ connect_each_object (gpointer key,
  * This function allows to control how the signal handlers are
  * going to be connected to their respective signals. It is meant
  * primarily for language bindings to allow resolving the function
- * names using the native API.
+ * names using the native API, but it can also be used on platforms
+ * that do not support GModule.
  *
  * Applications should use clutter_script_connect_signals().
  *
@@ -1141,9 +1148,10 @@ clutter_script_lookup_filename (ClutterScript *script,
  * Note: this function does not increment the reference count of the
  * objects it returns.
  *
- * Return value: (transfer container) (element-type GObject): a list of #GObject<!-- -->s,
- *   or %NULL. The objects are owned by the #ClutterScript instance. Use g_list_free() on the
- *   returned value when done.
+ * Return value: (transfer container) (element-type GObject.Object): a list
+ *   of #GObject<!-- -->s, or %NULL. The objects are owned by the
+ *   #ClutterScript instance. Use g_list_free() on the returned list when
+ *   done.
  *
  * Since: 0.8.2
  */
