@@ -144,29 +144,32 @@ enum
 {
   PROP_0,
 
-  PROP_SHADER_TYPE
+  PROP_SHADER_TYPE,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 G_DEFINE_ABSTRACT_TYPE (ClutterShaderEffect,
                         clutter_shader_effect,
                         CLUTTER_TYPE_OFFSCREEN_EFFECT);
 
 static inline void
-clutter_shader_effect_clear (ClutterShaderEffect *effect,
+clutter_shader_effect_clear (ClutterShaderEffect *self,
                              gboolean             reset_uniforms)
 {
-  ClutterShaderEffectPrivate *priv = effect->priv;
+  ClutterShaderEffectPrivate *priv = self->priv;
 
-  if (priv->shader != COGL_INVALID_HANDLE)
-    {
-      cogl_handle_unref (priv->shader);
-      priv->shader = COGL_INVALID_HANDLE;
-    }
+  if (priv->shader != COGL_INVALID_HANDLE && !priv->is_compiled)
+    cogl_handle_unref (priv->shader);
 
   if (priv->program != COGL_INVALID_HANDLE)
     {
       cogl_handle_unref (priv->program);
+
       priv->program = COGL_INVALID_HANDLE;
+      priv->shader = COGL_INVALID_HANDLE;
     }
 
   if (reset_uniforms && priv->uniforms != NULL)
@@ -208,6 +211,9 @@ clutter_shader_effect_update_uniforms (ClutterShaderEffect *effect)
   gpointer key, value;
   gsize size;
 
+  if (priv->program == COGL_INVALID_HANDLE)
+    return;
+
   if (priv->uniforms == NULL)
     return;
 
@@ -226,33 +232,44 @@ clutter_shader_effect_update_uniforms (ClutterShaderEffect *effect)
           const GLfloat *floats;
 
           floats = clutter_value_get_shader_float (&uniform->value, &size);
-          cogl_program_uniform_float (uniform->location, size, 1, floats);
+          cogl_program_set_uniform_float (priv->program, uniform->location,
+                                          size, 1,
+                                          floats);
         }
       else if (CLUTTER_VALUE_HOLDS_SHADER_INT (&uniform->value))
         {
           const GLint *ints;
 
           ints = clutter_value_get_shader_int (&uniform->value, &size);
-          cogl_program_uniform_int (uniform->location, size, 1, ints);
+          cogl_program_set_uniform_int (priv->program, uniform->location,
+                                        size, 1,
+                                        ints);
         }
       else if (CLUTTER_VALUE_HOLDS_SHADER_MATRIX (&uniform->value))
         {
           const GLfloat *matrix;
 
           matrix = clutter_value_get_shader_matrix (&uniform->value, &size);
-          cogl_program_uniform_matrix (uniform->location, size, 1, FALSE, matrix);
+          cogl_program_set_uniform_matrix (priv->program, uniform->location,
+                                           size, 1,
+                                           FALSE,
+                                           matrix);
         }
       else if (G_VALUE_HOLDS_FLOAT (&uniform->value))
         {
           const GLfloat float_val = g_value_get_float (&uniform->value);
 
-          cogl_program_uniform_float (uniform->location, 1, 1, &float_val);
+          cogl_program_set_uniform_float (priv->program, uniform->location,
+                                          1, 1,
+                                          &float_val);
         }
       else if (G_VALUE_HOLDS_INT (&uniform->value))
         {
           const GLint int_val = g_value_get_int (&uniform->value);
 
-          cogl_program_uniform_int (uniform->location, 1, 1, &int_val);
+          cogl_program_set_uniform_int (priv->program, uniform->location,
+                                        1, 1,
+                                        &int_val);
         }
       else
         g_warning ("Invalid uniform of type '%s' for name '%s'",
@@ -320,16 +337,17 @@ clutter_shader_effect_paint_target (ClutterOffscreenEffect *effect)
 {
   ClutterShaderEffectPrivate *priv = CLUTTER_SHADER_EFFECT (effect)->priv;
   ClutterOffscreenEffectClass *parent;
+  CoglHandle material;
 
   /* we haven't been prepared or we don't have support for
    * GLSL shaders in Clutter
    */
   if (priv->program == COGL_INVALID_HANDLE ||
       priv->shader == COGL_INVALID_HANDLE)
-    return;
+    goto out;
 
   if (!priv->source_set)
-    return;
+    goto out;
 
   if (!priv->is_compiled)
     {
@@ -349,10 +367,12 @@ clutter_shader_effect_paint_target (ClutterOffscreenEffect *effect)
           cogl_handle_unref (priv->program);
           priv->shader = COGL_INVALID_HANDLE;
 
-          return;
+          goto out;
         }
 
       cogl_program_attach_shader (priv->program, priv->shader);
+      cogl_handle_unref (priv->shader);
+
       cogl_program_link (priv->program);
 
       priv->is_compiled = TRUE;
@@ -361,17 +381,17 @@ clutter_shader_effect_paint_target (ClutterOffscreenEffect *effect)
   CLUTTER_NOTE (SHADER, "Applying the shader effect of type '%s'",
                 G_OBJECT_TYPE_NAME (effect));
 
-  /* set the shader */
-  cogl_program_use (priv->program);
-
   clutter_shader_effect_update_uniforms (CLUTTER_SHADER_EFFECT (effect));
 
+  /* associate the program to the offscreen target material */
+  material = clutter_offscreen_effect_get_target (effect);
+  cogl_material_set_user_program (material, priv->program);
+
+out:
   /* paint the offscreen buffer */
   parent = CLUTTER_OFFSCREEN_EFFECT_CLASS (clutter_shader_effect_parent_class);
   parent->paint_target (effect);
 
-  /* unset the shader */
-  cogl_program_use (COGL_INVALID_HANDLE);
 }
 
 static void
@@ -429,12 +449,13 @@ clutter_shader_effect_class_init (ClutterShaderEffectClass *klass)
    * Since: 1.4
    */
   pspec = g_param_spec_enum ("shader-type",
-                             "Shader Type",
-                             "The type of shader used",
+                             P_("Shader Type"),
+                             P_("The type of shader used"),
                              CLUTTER_TYPE_SHADER_TYPE,
                              CLUTTER_FRAGMENT_SHADER,
                              CLUTTER_PARAM_WRITABLE |
                              G_PARAM_CONSTRUCT_ONLY);
+  obj_props[PROP_SHADER_TYPE] = pspec;
   g_object_class_install_property (gobject_class, PROP_SHADER_TYPE, pspec);
 
   meta_class->set_actor = clutter_shader_effect_set_actor;
@@ -514,6 +535,7 @@ shader_uniform_new (const gchar  *name,
   retval->name = g_strdup (name);
   retval->type = G_VALUE_TYPE (value);
   retval->location = -1;
+  retval->value = (GValue){0};
 
   g_value_init (&retval->value, retval->type);
   g_value_copy (value, &retval->value);
