@@ -118,9 +118,13 @@ enum
 {
   PROP_0,
   PROP_DURATION,
-  PROP_STATE
+  PROP_STATE,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 enum
 {
   COMPLETED,
@@ -238,6 +242,8 @@ clutter_state_key_free (gpointer clutter_state_key)
                            key->target_state->clutter_state);
     }
 
+  g_value_unset (&key->value);
+
   g_object_unref (key->alpha);
   g_object_unref (key->interval);
 
@@ -263,6 +269,8 @@ clutter_state_remove_key_internal (ClutterState *this,
   if (source_state_name)
     source_state = clutter_state_fetch_state (this, source_state_name, FALSE);
 
+again_from_start:
+
   if (target_state_name != NULL)
     state_list = g_list_append (NULL, (gpointer) target_state_name);
   else
@@ -272,31 +280,36 @@ clutter_state_remove_key_internal (ClutterState *this,
     {
       State *target_state;
       target_state = clutter_state_fetch_state (this, s->data, FALSE);
-
+      /* Go through each TargetState */
       if (target_state)
         {
           GList *k;
-again:
+again_for_target_state:
           for (k = target_state->keys; k != NULL; k = k->next)
             {
               ClutterStateKey *key = k->data;
 
+              /* Check if each key matches query */
               if (   (object == NULL        || (object == key->object))
                   && (source_state == NULL  || (source_state == key->source_state))
                   && (property_name == NULL || ((property_name == key->property_name))))
                 {
+                  /* Remove matching key */
                   target_state->keys = g_list_remove (target_state->keys, key);
-
-                  if (target_state->keys == NULL)
-                    {
-                      /* no more keys, so remove this state */
-                      clutter_state_remove_key (this, s->data, NULL, NULL, NULL);
-                      g_hash_table_remove (this->priv->states, s->data);
-                    }
-
                   key->is_inert = is_inert;
                   clutter_state_key_free (key);
-                  goto again;
+
+                  /* no more keys with transitions to this target_state*/
+                  if (target_state->keys == NULL)
+                    {
+                      /* remove any keys that exist that uses this state as a source */
+                      clutter_state_remove_key (this, s->data, NULL, NULL, NULL);
+
+                      g_hash_table_remove (this->priv->states, s->data);
+                      goto again_from_start; /* we have just freed State *target_state, so
+                                                need to restart removal */
+                    }
+                  goto again_for_target_state;
                 }
             }
         }
@@ -477,7 +490,7 @@ clutter_state_change (ClutterState *state,
   priv->source_state_name = priv->target_state_name;
   priv->target_state_name = target_state_name;
 
-  g_object_notify (G_OBJECT (state), "state");
+  _clutter_notify_by_pspec (G_OBJECT (state), obj_props[PROP_STATE]);
 
   duration = clutter_state_get_duration (state,
                                          priv->source_state_name,
@@ -555,7 +568,9 @@ clutter_state_change (ClutterState *state,
  * The state will animate during its transition, see
  * #clutter_state_warp_to_state for animation-free state switching.
  *
- * Return value: the #ClutterTimeline that drives the state transition
+ * Return value: (transfer none): the #ClutterTimeline that drives the
+ *   state transition. The returned timeline is owned by the #ClutterState
+ *   and it should not be unreferenced
  *
  * Since: 1.4
  */
@@ -575,7 +590,9 @@ clutter_state_set_state (ClutterState *state,
  *
  * Change to the specified target state immediately with no animation.
  *
- * Return value: the #ClutterTimeline that drives the state transition
+ * Return value: (transfer none): the #ClutterTimeline that drives the
+ *   state transition. The returned timeline is owned by the #ClutterState
+ *   and it should not be unreferenced
  *
  * Since: 1.4
  */
@@ -726,12 +743,15 @@ clutter_state_set (ClutterState *state,
       GParamSpec *pspec;
       GValue value = { 0, };
       gchar *error = NULL;
-      const gchar *real_property_name = property_name;
+      gboolean is_delayed = FALSE;
 
       if (g_str_has_prefix (property_name, "delayed::"))
-        real_property_name = strstr (property_name, "::") + 2;
+        {
+          property_name = strstr (property_name, "::") + 2;
+          is_delayed = TRUE;
+        }
 
-      pspec = get_property_from_object (object, real_property_name);
+      pspec = get_property_from_object (object, property_name);
       if (pspec == NULL)
         break;
 
@@ -751,7 +771,7 @@ clutter_state_set (ClutterState *state,
           break;
         }
 
-      if (g_str_has_prefix (property_name, "delayed::"))
+      if (is_delayed)
         {
           gdouble pre_delay = va_arg (args, gdouble);
           gdouble post_delay = va_arg (args, gdouble);
@@ -760,7 +780,7 @@ clutter_state_set (ClutterState *state,
                                  source_state_name,
                                  target_state_name,
                                  object,
-                                 real_property_name,
+                                 property_name,
                                  mode,
                                  &value,
                                  pre_delay,
@@ -779,6 +799,8 @@ clutter_state_set (ClutterState *state,
                                  0.0);
         }
 
+      g_value_unset (&value);
+
       object = va_arg (args, gpointer);
       if (object != NULL)
         {
@@ -870,8 +892,8 @@ clutter_state_fetch_state (ClutterState *state,
  * Sets one specific end key for a state_name, object, property_name
  * combination.
  *
- * Return value: the #ClutterState instance, allowing chaining of
- *   multiple calls
+ * Return value: (transfer none): the #ClutterState instance, allowing
+ *   chaining of multiple calls
  *
  * Since: 1.4
  */
@@ -959,7 +981,7 @@ clutter_state_get_states (ClutterState *state)
  * Returns a list of pointers to opaque structures with accessor functions
  * that describe the keys added to an animator.
  *
- * Return value: (transfer container) (element-type ClutterStateKey): a
+ * Return value: (transfer container) (element-type Clutter.StateKey): a
  *   newly allocated #GList of #ClutterStateKey<!-- -->s. The contents of
  *   the returned list are owned by the #ClutterState and should not be
  *   modified or freed. Use g_list_free() to free the resources allocated
@@ -1160,6 +1182,7 @@ clutter_state_class_init (ClutterStateClass *klass)
                                P_("Currently set state, (transition to this state might not be complete)"),
                                NULL,
                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_STATE] = pspec;
   g_object_class_install_property (gobject_class, PROP_STATE, pspec);
 
   /**
@@ -1173,6 +1196,7 @@ clutter_state_class_init (ClutterStateClass *klass)
                              P_("Default transition duration"),
                              0, 86400000, 1000,
                              CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_DURATION] = pspec;
   g_object_class_install_property (gobject_class, PROP_DURATION, pspec);
 }
 
@@ -1403,20 +1427,26 @@ clutter_state_key_get_mode (const ClutterStateKey *state_key)
  * @state_key: a #ClutterStateKey
  * @value: a #GValue initialized with the correct type for the @state_key
  *
- * Copies of the value for a #ClutterStateKey into @value
+ * Retrieves a copy of the value for a #ClutterStateKey.
  *
  * The #GValue needs to be already initialized for the value type
- * of the property or to a transformable type
+ * of the property or to a type that allow transformation from the value
+ * type of the key.
+ *
+ * Use g_value_unset() when done.
+ *
+ * Return value: %TRUE if the value was successfully retrieved,
+ *   and %FALSE otherwise
  *
  * Since: 1.4
  */
-void
+gboolean
 clutter_state_key_get_value (const ClutterStateKey *state_key,
                              GValue                *value)
 {
-  g_return_if_fail (state_key != NULL);
-  g_return_if_fail (value != NULL);
-  g_return_if_fail (G_VALUE_TYPE (value) != G_TYPE_INVALID);
+  g_return_val_if_fail (state_key != NULL, FALSE);
+  g_return_val_if_fail (value != NULL, FALSE);
+  g_return_val_if_fail (G_VALUE_TYPE (value) != G_TYPE_INVALID, FALSE);
 
   if (!g_type_is_a (G_VALUE_TYPE (&state_key->value), G_VALUE_TYPE (value)))
     {
@@ -1424,14 +1454,14 @@ clutter_state_key_get_value (const ClutterStateKey *state_key,
                                    G_VALUE_TYPE (value)))
         {
           g_value_copy (&state_key->value, value);
-          return;
+          return TRUE;
         }
 
       if (g_value_type_transformable (G_VALUE_TYPE (&state_key->value),
                                       G_VALUE_TYPE (value)))
         {
           if (g_value_transform (&state_key->value, value))
-            return;
+            return TRUE;
         }
 
       g_warning ("%s: Unable to convert from %s to %s for the "
@@ -1441,9 +1471,13 @@ clutter_state_key_get_value (const ClutterStateKey *state_key,
                  g_type_name (G_VALUE_TYPE (value)),
                  state_key->property_name,
                  G_OBJECT_TYPE_NAME (state_key->object));
+
+      return FALSE;
     }
   else
     g_value_copy (&state_key->value, value);
+
+  return TRUE;
 }
 
 /**
@@ -1708,7 +1742,7 @@ parse_state_transition (JsonArray *array,
   State *source_state, *target_state;
   JsonArray *keys;
   GSList *valid_keys = NULL;
-  GList *k;
+  GList *array_keys, *k;
 
   if (JSON_NODE_TYPE (element) != JSON_NODE_OBJECT)
     {
@@ -1780,9 +1814,8 @@ parse_state_transition (JsonArray *array,
   else
     g_value_init (clos->value, G_TYPE_POINTER);
 
-  for (k = json_array_get_elements (keys);
-       k != NULL;
-       k = k->next)
+  array_keys = json_array_get_elements (keys);
+  for (k = array_keys; k != NULL; k = k->next)
     {
       JsonNode *node = k->data;
       JsonArray *key = json_node_get_array (node);
@@ -1860,6 +1893,8 @@ parse_state_transition (JsonArray *array,
       valid_keys = g_slist_prepend (valid_keys, state_key);
     }
 
+  g_list_free (array_keys);
+
   g_value_set_pointer (clos->value, g_slist_reverse (valid_keys));
 
   clos->result = TRUE;
