@@ -155,6 +155,9 @@ initialize_texture_units (CoglGles2Wrapper *w)
                                 GL_SRC_COLOR) );
       GE( _cogl_wrap_glTexEnvi (GL_TEXTURE_ENV, GL_OPERAND1_ALPHA,
                                 GL_SRC_COLOR) );
+
+      GE( _cogl_wrap_glTexEnvi (GL_POINT_SPRITE, GL_COORD_REPLACE,
+                                GL_FALSE) );
     }
 
   GE( _cogl_wrap_glMatrixMode ((GLenum) prev_mode));
@@ -194,6 +197,9 @@ _cogl_gles2_wrapper_init (CoglGles2Wrapper *wrapper)
   _cogl_wrap_glDisable (GL_ALPHA_TEST);
   _cogl_wrap_glAlphaFunc (GL_ALWAYS, 0.0f);
 
+  /* Initialize the point size */
+  _cogl_wrap_glPointSize (1.0f);
+
   initialize_texture_units (wrapper);
 }
 
@@ -246,30 +252,43 @@ cogl_gles2_settings_equal (const CoglGles2WrapperSettings *a,
   /* Compare layer combine operation for each active unit */
   if (fragment_tests)
     for (i = 0; i < COGL_GLES2_MAX_TEXTURE_UNITS; i++)
-      if (COGL_GLES2_TEXTURE_UNIT_IS_ENABLED (a->texture_units, i))
-        {
-          const CoglGles2WrapperTexEnv *tex_env_a = a->tex_env + i;
-          const CoglGles2WrapperTexEnv *tex_env_b = b->tex_env + i;
-          int arg, n_args;
-          GLenum func;
+      {
+        if (!!COGL_GLES2_TEXTURE_UNIT_IS_ENABLED (a->texture_units, i) !=
+            !!COGL_GLES2_TEXTURE_UNIT_IS_ENABLED (b->texture_units, i))
+          return FALSE;
 
-          func = tex_env_a->texture_combine_rgb_func;
+        if (COGL_GLES2_TEXTURE_UNIT_IS_ENABLED (a->texture_units, i))
+          {
+            const CoglGles2WrapperTexEnv *tex_env_a = a->tex_env + i;
+            const CoglGles2WrapperTexEnv *tex_env_b = b->tex_env + i;
+            int arg, n_args;
+            GLenum func;
 
-          if (func != tex_env_b->texture_combine_rgb_func)
-            return FALSE;
+            if (tex_env_a->texture_target != tex_env_b->texture_target)
+              return FALSE;
 
-          n_args = cogl_gles2_get_n_args_for_combine_func (func);
+            if (tex_env_a->point_sprite_coords !=
+                tex_env_b->point_sprite_coords)
+              return FALSE;
 
-          for (arg = 0; arg < n_args; arg++)
-            {
-              if (tex_env_a->texture_combine_rgb_src[arg] !=
-                  tex_env_b->texture_combine_rgb_src[arg])
-                return FALSE;
-              if (tex_env_a->texture_combine_rgb_op[arg] !=
-                  tex_env_b->texture_combine_rgb_op[arg])
-                return FALSE;
-            }
-        }
+            func = tex_env_a->texture_combine_rgb_func;
+
+            if (func != tex_env_b->texture_combine_rgb_func)
+              return FALSE;
+
+            n_args = cogl_gles2_get_n_args_for_combine_func (func);
+
+            for (arg = 0; arg < n_args; arg++)
+              {
+                if (tex_env_a->texture_combine_rgb_src[arg] !=
+                    tex_env_b->texture_combine_rgb_src[arg])
+                  return FALSE;
+                if (tex_env_a->texture_combine_rgb_op[arg] !=
+                    tex_env_b->texture_combine_rgb_op[arg])
+                  return FALSE;
+              }
+          }
+      }
 
   return TRUE;
 }
@@ -326,7 +345,8 @@ cogl_gles2_get_vertex_shader (const CoglGles2WrapperSettings *settings)
   g_string_append (shader_source, _cogl_fixed_vertex_shader_main_start);
 
   for (i = 0; i < COGL_GLES2_MAX_TEXTURE_UNITS; i++)
-    if (COGL_GLES2_TEXTURE_UNIT_IS_ENABLED (settings->texture_units, i))
+    if (COGL_GLES2_TEXTURE_UNIT_IS_ENABLED (settings->texture_units, i) &&
+        !settings->tex_env[i].point_sprite_coords)
       {
         g_string_append_printf (shader_source,
                                 "transformed_tex_coord = "
@@ -383,6 +403,31 @@ cogl_gles2_get_vertex_shader (const CoglGles2WrapperSettings *settings)
   return shader;
 }
 
+static void
+cogl_gles2_add_texture_lookup (int unit,
+                               const char *swizzle,
+                               GString *shader_source)
+{
+  _COGL_GET_GLES2_WRAPPER (w, NO_RETVAL);
+
+  if (w->settings.tex_env[unit].texture_target == GL_TEXTURE_3D_OES)
+    g_string_append_printf (shader_source, "texture3D (texture_unit[%d], ",
+                            unit);
+  else
+    g_string_append_printf (shader_source, "texture2D (texture_unit[%d], ",
+                            unit);
+
+  /* If point sprite coord generation is being used then divert to the
+     built-in varying var for that instead of the texture
+     coordinates */
+  if (w->settings.tex_env[unit].point_sprite_coords)
+    g_string_append (shader_source, "gl_PointCoord");
+  else
+    g_string_append_printf (shader_source, "tex_coord[%d]", unit);
+
+  g_string_append_printf (shader_source, ").%s", swizzle);
+}
+
 static void
 cogl_gles2_add_arg (int unit,
                     GLenum src,
@@ -410,9 +455,7 @@ cogl_gles2_add_arg (int unit,
   switch (src)
     {
     case GL_TEXTURE:
-      g_string_append_printf (shader_source,
-                              "texture2D (texture_unit[%d], tex_coord[%d]).%s",
-                              unit, unit, swizzle);
+      cogl_gles2_add_texture_lookup (unit, swizzle, shader_source);
       break;
 
     case GL_CONSTANT:
@@ -434,12 +477,8 @@ cogl_gles2_add_arg (int unit,
     default:
       if (src >= GL_TEXTURE0 &&
           src < GL_TEXTURE0 + COGL_GLES2_MAX_TEXTURE_UNITS)
-        g_string_append_printf (shader_source,
-                                "texture2D (texture_unit[%d], "
-                                "tex_coord[%d]).%s",
-                                src - GL_TEXTURE0,
-                                src - GL_TEXTURE0,
-                                swizzle);
+        cogl_gles2_add_texture_lookup (src - GL_TEXTURE0,
+                                       swizzle, shader_source);
       break;
     }
 
@@ -772,6 +811,9 @@ cogl_gles2_wrapper_get_locations (GLuint program,
 
   uniforms->alpha_test_ref_uniform
     = glGetUniformLocation (program, "alpha_test_ref");
+
+  uniforms->point_size_uniform
+    = glGetUniformLocation (program, "point_size");
 }
 
 static void
@@ -888,8 +930,8 @@ cogl_gles2_wrapper_get_program (const CoglGles2WrapperSettings *settings)
 
   /* We haven't tried to get a location for any of the custom uniforms
      yet */
-  for (i = 0; i < COGL_GLES2_NUM_CUSTOM_UNIFORMS; i++)
-    program->custom_uniforms[i] = COGL_GLES2_UNBOUND_CUSTOM_UNIFORM;
+  for (i = 0; i < COGL_PROGRAM_NUM_CUSTOM_UNIFORMS; i++)
+    program->custom_gl_uniforms[i] = COGL_PROGRAM_UNBOUND_CUSTOM_UNIFORM;
 
   w->compiled_programs = g_slist_append (w->compiled_programs, program);
 
@@ -900,7 +942,6 @@ void
 _cogl_gles2_wrapper_deinit (CoglGles2Wrapper *wrapper)
 {
   GSList *node, *next;
-  int i;
 
   for (node = wrapper->compiled_programs; node; node = next)
     {
@@ -925,10 +966,6 @@ _cogl_gles2_wrapper_deinit (CoglGles2Wrapper *wrapper)
       g_slist_free1 (node);
     }
   wrapper->compiled_fragment_shaders = NULL;
-
-  for (i = 0; i < COGL_GLES2_NUM_CUSTOM_UNIFORMS; i++)
-    if (wrapper->custom_uniforms[i].count > 1)
-      g_free (wrapper->custom_uniforms[i].v.array);
 }
 
 static void
@@ -1138,6 +1175,7 @@ static void
 _cogl_wrap_prepare_for_draw (void)
 {
   CoglGles2WrapperProgram *program;
+  guint32 dirty_custom_uniforms = 0;
 
   _COGL_GET_GLES2_WRAPPER (w, NO_RETVAL);
 
@@ -1158,12 +1196,18 @@ _cogl_wrap_prepare_for_draw (void)
 	  w->current_program = program;
 	  /* All of the uniforms are probably now out of date */
 	  w->dirty_uniforms = COGL_GLES2_DIRTY_ALL;
-	  w->dirty_custom_uniforms = (1 << COGL_GLES2_NUM_CUSTOM_UNIFORMS) - 1;
+	  dirty_custom_uniforms = (1 << COGL_PROGRAM_NUM_CUSTOM_UNIFORMS) - 1;
 	}
       w->settings_dirty = FALSE;
     }
   else
-    program = w->current_program;
+    {
+      CoglProgram *user_program =
+        _cogl_program_pointer_from_handle (w->settings.user_program);
+      if (user_program)
+        dirty_custom_uniforms = user_program->dirty_custom_uniforms;
+      program = w->current_program;
+    }
 
   /* Make sure all of the uniforms are up to date */
   if (w->dirty_uniforms)
@@ -1235,10 +1279,14 @@ _cogl_wrap_prepare_for_draw (void)
             }
         }
 
+      if ((w->dirty_uniforms & COGL_GLES2_DIRTY_POINT_SIZE))
+        glUniform1f (program->uniforms.point_size_uniform,
+                     w->point_size);
+
       w->dirty_uniforms = 0;
     }
 
-  if (w->dirty_custom_uniforms)
+  if (dirty_custom_uniforms)
     {
       int i;
 
@@ -1248,21 +1296,20 @@ _cogl_wrap_prepare_for_draw (void)
 	    = _cogl_program_pointer_from_handle (w->settings.user_program);
 	  const char *uniform_name;
 
-	  for (i = 0; i < COGL_GLES2_NUM_CUSTOM_UNIFORMS; i++)
-	    if ((w->dirty_custom_uniforms & (1 << i))
+	  for (i = 0; i < COGL_PROGRAM_NUM_CUSTOM_UNIFORMS; i++)
+	    if ((dirty_custom_uniforms & (1 << i))
 		&& (uniform_name = user_program->custom_uniform_names[i]))
 	      {
-		if (program->custom_uniforms[i]
-		    == COGL_GLES2_UNBOUND_CUSTOM_UNIFORM)
-		  program->custom_uniforms[i]
+		if (program->custom_gl_uniforms[i]
+		    == COGL_PROGRAM_UNBOUND_CUSTOM_UNIFORM)
+		  program->custom_gl_uniforms[i]
 		    = glGetUniformLocation (program->program, uniform_name);
-		if (program->custom_uniforms[i] >= 0)
-		  cogl_gles2_do_set_uniform (program->custom_uniforms[i],
-					     &w->custom_uniforms[i]);
+		if (program->custom_gl_uniforms[i] >= 0)
+		  cogl_gles2_do_set_uniform (program->custom_gl_uniforms[i],
+					     &user_program->custom_uniforms[i]);
 	      }
+          user_program->dirty_custom_uniforms = 0;
 	}
-
-      w->dirty_custom_uniforms = 0;
     }
 
   if (w->dirty_attribute_pointers
@@ -1340,14 +1387,14 @@ _cogl_wrap_glDrawElements (GLenum mode, GLsizei count, GLenum type,
 void
 _cogl_wrap_glTexEnvi (GLenum target, GLenum pname, GLint param)
 {
+  CoglGles2WrapperTexEnv *tex_env;
+
+  _COGL_GET_GLES2_WRAPPER (w, NO_RETVAL);
+
+  tex_env = w->settings.tex_env + w->active_texture_unit;
+
   if (target == GL_TEXTURE_ENV)
     {
-      CoglGles2WrapperTexEnv *tex_env;
-
-      _COGL_GET_GLES2_WRAPPER (w, NO_RETVAL);
-
-      tex_env = w->settings.tex_env + w->active_texture_unit;
-
       switch (pname)
         {
         case GL_COMBINE_RGB:
@@ -1378,6 +1425,17 @@ _cogl_wrap_glTexEnvi (GLenum target, GLenum pname, GLint param)
           break;
         }
 
+      w->settings_dirty = TRUE;
+    }
+  else if (target == GL_POINT_SPRITE)
+    {
+      switch (pname)
+        {
+        case GL_COORD_REPLACE:
+          tex_env->point_sprite_coords = param;
+          break;
+        }
+
       w->settings_dirty = TRUE;
     }
 }
@@ -1427,6 +1485,7 @@ _cogl_wrap_glEnable (GLenum cap)
   switch (cap)
     {
     case GL_TEXTURE_2D:
+    case GL_TEXTURE_3D_OES:
       if (!COGL_GLES2_TEXTURE_UNIT_IS_ENABLED (w->settings.texture_units,
                                                w->active_texture_unit))
         {
@@ -1435,6 +1494,12 @@ _cogl_wrap_glEnable (GLenum cap)
                                                TRUE);
           w->settings_dirty = TRUE;
         }
+      /* Keep track of the last enabled texture unit */
+      if (w->settings.tex_env[w->active_texture_unit].texture_target != cap)
+        {
+          w->settings.tex_env[w->active_texture_unit].texture_target = cap;
+          w->settings_dirty = TRUE;
+        }
       break;
 
     case GL_FOG:
@@ -1458,8 +1523,12 @@ _cogl_wrap_glDisable (GLenum cap)
   switch (cap)
     {
     case GL_TEXTURE_2D:
+    case GL_TEXTURE_3D_OES:
+      /* If this was the last enabled texture target then we'll
+         completely disable the unit */
       if (COGL_GLES2_TEXTURE_UNIT_IS_ENABLED (w->settings.texture_units,
-                                              w->active_texture_unit))
+                                              w->active_texture_unit) &&
+          w->settings.tex_env[w->active_texture_unit].texture_target == cap)
         {
           COGL_GLES2_TEXTURE_UNIT_SET_ENABLED (w->settings.texture_units,
                                                w->active_texture_unit,
@@ -1688,6 +1757,15 @@ _cogl_wrap_glMaterialfv (GLenum face, GLenum pname, const GLfloat *params)
      function can't do anything */
 }
 
+void
+_cogl_wrap_glPointSize (GLfloat size)
+{
+  _COGL_GET_GLES2_WRAPPER (w, NO_RETVAL);
+
+  w->point_size = size;
+  w->dirty_uniforms |= COGL_GLES2_DIRTY_POINT_SIZE;
+}
+
 void
 _cogl_gles2_clear_cache_for_program (CoglHandle user_program)
 {
