@@ -39,22 +39,6 @@ clutter_x11_xinput_event_types_get_type(void) {
   return etype;
 }
 
-GType
-clutter_x11_input_device_type_get_type(void) {
-  static GType etype = 0;
-  if (G_UNLIKELY (!etype))
-    {
-      static const GEnumValue values[] = {
-        { CLUTTER_X11_XINPUT_POINTER_DEVICE, "CLUTTER_X11_XINPUT_POINTER_DEVICE", "pointer-device" },
-        { CLUTTER_X11_XINPUT_KEYBOARD_DEVICE, "CLUTTER_X11_XINPUT_KEYBOARD_DEVICE", "keyboard-device" },
-        { CLUTTER_X11_XINPUT_EXTENSION_DEVICE, "CLUTTER_X11_XINPUT_EXTENSION_DEVICE", "extension-device" },
-        { 0, NULL, NULL }
-      };
-      etype = g_enum_register_static (g_intern_static_string ("ClutterX11InputDeviceType"), values);
-    }
-  return etype;
-}
-
 
 /* Generated data ends here */
 
