@@ -1,3 +1,7 @@
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <gmodule.h>
 #include <clutter/clutter.h>
 
@@ -10,6 +14,10 @@
 #include <stdlib.h>
 #include <glib.h>
 
+#ifdef HAVE_CLUTTER_GLX
+#include "clutter/x11/clutter-x11.h"
+#endif
+
 #define NHANDS  6
 
 typedef struct SuperOH
@@ -79,13 +87,13 @@ input_cb (ClutterActor *stage,
       g_print ("*** key press event (key:%c) ***\n",
 	       clutter_event_get_key_symbol (event));
 
-      if (clutter_event_get_key_symbol (event) == CLUTTER_q)
+      if (clutter_event_get_key_symbol (event) == CLUTTER_KEY_q)
         {
 	  clutter_main_quit ();
 
           return TRUE;
         }
-      else if (clutter_event_get_key_symbol (event) == CLUTTER_r)
+      else if (clutter_event_get_key_symbol (event) == CLUTTER_KEY_r)
         {
           gint i;
 
@@ -195,6 +203,10 @@ test_paint_wrapper_main (int argc, char *argv[])
 
   error = NULL;
 
+#ifdef HAVE_CLUTTER_GLX
+  clutter_x11_set_use_argb_visual (TRUE);
+#endif
+
   clutter_init_with_args (&argc, &argv,
                           NULL,
                           super_oh_entries,
